@@ -14,9 +14,8 @@ use super::{Properties, Property, PropertyName, PropertyType, Tracker, Widget, W
 impl Parse for Tracker {
     fn parse(input: ParseStream) -> Result<Self> {
         let bool_fn = input.parse()?;
-        let _comma: Token![,] = input.parse()?;
-        let mut update_fns = vec![input.parse()?];
 
+        let mut update_fns = Vec::new();
         while !input.is_empty() {
             let _comma: Token![,] = input.parse()?;
             // allow comma at the end of the macro
@@ -58,6 +57,7 @@ impl Parse for Property {
                     return Ok(Property {
                         name,
                         ty: PropertyType::Factory(paren_input.parse()?),
+                        generics: None,
                         optional_assign,
                         iterative,
                         args: None,
@@ -83,6 +83,12 @@ impl Parse for Property {
             None
         };
 
+        let generics = if input.peek(Token![<]) {
+            Some(input.parse()?)
+        } else {
+            None
+        };
+
         // look for event handlers: property(a, ...) => move |a, ...| { ... }
         let ty = if input.peek(Token! [=>]) {
             let _arrow: Token![=>] = input.parse()?;
@@ -168,6 +174,7 @@ impl Parse for Property {
             Ok(Property {
                 name,
                 ty,
+                generics,
                 args,
                 optional_assign,
                 iterative,
